/*	Copyright  (c)	Günter Woigk  2004
  					mailto:kio@little-bat.de

 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

 	Permission to use, copy, modify, distribute, and sell this software and
 	its documentation for any purpose is hereby granted without fee, provided
 	that the above copyright notice appear in all copies and that both that
 	copyright notice and this permission notice appear in supporting
 	documentation, and that the name of the copyright holder not be used
 	in advertising or publicity pertaining to distribution of the software
 	without specific, written prior permission.  The copyright holder makes no
 	representations about the suitability of this software for any purpose.
 	It is provided "as is" without express or implied warranty.

 	THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 	INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 	EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 	CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 	DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 	TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 	PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef Item_h
#define	Item_h

#include "kio/kio.h"
#include "DspTime.h"


@class Machine;


class Item
{	
protected:
	Machine*	machine;
	cstr		name;

private:		
				Item			( Item const& );		// prohibit
	Item&		operator=		( Item const& );		// prohibit

public:
		/*	create: 
			if o_addr!=NULL then register for Output()
			if i_addr!=NULL  then register for Input()
			The creator must not yet assume that the machine is set up properly or that any other item already exist.
			Setup the item as far as possible. PowerOn() will be called later.
		*/
				Item			( Machine*, cstr name, cstr o_addr, cstr i_addr );
				Item			( Machine*, cstr name );

		/*	destroy self
		*/
virtual			~Item			( );


		/*	Information
		*/
cstr			GetName			( )				{ return name; }
	

//	===================================================================
//			Interface definition:
//	===================================================================

		/*	Time t  is biased to dsp_buffer_start  ((== system_time))
					=> always in range 0 .. ticks_per_dsp_buffer		((plus allowed overflow into stitching buffer))
					=> real time = t + system_time

			long cc is biased to machine->cc_frame_start
					=> always in range 0 .. machine->ula->cc_per_frame  ((plus allowed overflow for the last instruction))
					=> virtual machine time = cc + machine->cc_frame_start

			if the cpu is throttled down heavily, then cc may overshoot a lot beyond dsp_buffer end, 
			but Time t is always forced legal, that is: inside dsp_buffer+stitching

			Either or both may happily be ignored.
			Any unused routine may implement a dummy stub.
			Concurrent callbacks are prevented by the machine.
		*/


		/*	initialize self for power-on
			called after all items have been created
			discard any pending processing
			setup references to other items
		*/
virtual void	PowerOn			( Time t, long cc );


		/*	cpu reset button pressed
			finish pending processing up to t/cc and then reset.
			ignore the callback if the item is not connected to the cpu reset.
		*/
virtual void	Reset			( Time t, long cc );


		/*	handle Output
			called if address matches registered o_addr address
		*/
virtual	void	Output			( Time t, long cc, ushort address, uchar byte );


		/*	handle Input
			called if address matches registered i_addr address
			note: return 0xFF if address did actually not match (if valid addresses could not be sufficiently be described with bits & mask)
		*/
virtual	uchar	Input			( Time t, long cc, ushort address );


		/*	DSP Buffer End Callback
			sound emitters must pad their last sample to the dsp buffer end
			sound i/o items must decrement their time base by ticks_per_dsp_buffer
			this call may be considered as a 'broadcast' input/output
		*/
virtual void	DspBufferEnd	( Time t, long cc );


		/*	cpu clock predivider was changed by user
			the time base for the virtual machine is the ula clock.
			for faster cpu emulation actually the cpu clock is used.
			Therefore items measuring timing in virtual time space must know
			when the cpu clock predivider is changed to adjust their timing parameters…
		*/
virtual	void	CpuPredividerChanged	( int old_predivider, int new_predivider );
};


#endif






